@@ -5,10 +5,10 @@ Supported devices:
 - EVGA CLC (120 CL12, 240, 280 or 360); modern generic Asetek 690LC
 - NZXT Kraken X (X31, X41 or X61); legacy generic Asetek 690LC
 - NZXT Kraken X (X40 or X60); legacy generic Asetek 690LC
-- Corsair H80i GT, H100i GTX or H110i GTX
-- Corsair H80i v2, H100i v2 or H115i
+- Corsair Hydro H80i GT, H100i GTX or H110i GTX
+- Corsair Hydro H80i v2, H100i v2 or H115i
 
-Copyright (C) 2018–2021  Jonas Malaco and contributors
+Copyright Jonas Malaco and contributors
 
 Incorporates or uses as reference work by Kristóf Jakab, Sean Nelson
 and Chris Griffith.
@@ -49,9 +49,7 @@ _MIN_PUMP_SPEED_CODE = 0x32
 _MAX_PUMP_SPEED_CODE = 0x42
 _READ_ENDPOINT = 0x82
 _READ_LENGTH = 32
-_READ_TIMEOUT = 2000
 _WRITE_ENDPOINT = 0x2
-_WRITE_TIMEOUT = 2000
 
 _LEGACY_FIXED_SPEED_CHANNELS = {    # (message type, minimum duty, maximum duty)
     'fan':  (_CMD_OVERRIDE, 0, 100),
@@ -74,9 +72,27 @@ _UNKNOWN_OPEN_VALUE = 0xffff
 _USBXPRESS = usb.util.CTRL_OUT | usb.util.CTRL_TYPE_VENDOR | usb.util.CTRL_RECIPIENT_DEVICE
 
 
-class _CommonAsetekDriver(UsbDriver):
+class _Base690Lc(UsbDriver):
     """Common methods for Asetek 690LC devices."""
 
+    _LEGACY_690LC = False
+
+    @classmethod
+    def probe(cls, handle, legacy_690lc=False, **kwargs):
+        """Probe `handle` and yield corresponding driver instances."""
+        if legacy_690lc != cls._LEGACY_690LC:
+            return
+        yield from super().probe(handle, **kwargs)
+
+    @classmethod
+    def find_supported_devices(cls, **kwargs):
+        """Find devices specifically compatible with this driver.
+
+        Automatically sets the appropriate value for `legacy_690lc`.
+        """
+
+        return super().find_supported_devices(legacy_690lc=cls._LEGACY_690LC, **kwargs)
+
     def _configure_flow_control(self, clear_to_send):
         """Set the software clear-to-send flow control policy for device."""
         _LOGGER.debug('set clear to send = %s', clear_to_send)
@@ -92,7 +108,7 @@ class _CommonAsetekDriver(UsbDriver):
         self.device.ctrl_transfer(_USBXPRESS, _USBXPRESS_REQUEST, _USBXPRESS_FLUSH_BUFFERS)
 
     def _write(self, data):
-        self.device.write(_WRITE_ENDPOINT, data, _WRITE_TIMEOUT)
+        self.device.write(_WRITE_ENDPOINT, data)
 
     def _end_transaction_and_read(self):
         """End the transaction by reading from the device.
@@ -104,7 +120,7 @@ class _CommonAsetekDriver(UsbDriver):
         successfully follow this approach.
         """
 
-        msg = self.device.read(_READ_ENDPOINT, _READ_LENGTH, _READ_TIMEOUT)
+        msg = self.device.read(_READ_ENDPOINT, _READ_LENGTH)
         self.device.release()
         return msg
 
@@ -115,16 +131,16 @@ class _CommonAsetekDriver(UsbDriver):
                     + [alert_temp, interval1, interval2, not blackout, fading,
                        blinking, enable_alert, 0x00, 0x01])
 
-    def _prepare_profile(self, profile, min_duty, max_duty):
+    def _prepare_profile(self, profile, min_duty, max_duty, max_points):
         opt = list(profile)
         size = len(opt)
         if size < 1:
             raise ValueError('at least one PWM point required')
         elif size > _MAX_PROFILE_POINTS:
-            raise ValueError(f'too many PWM points ({size}), only 6 supported')
+            raise ValueError(f'too many PWM points ({size}), only {max_points} supported')
         for i, (temp, duty) in enumerate(opt):
             opt[i] = (temp, clamp(duty, min_duty, max_duty))
-        missing = _MAX_PROFILE_POINTS - size
+        missing = max_points - size
         if missing:
             # Some issues were observed when padding with (0°C, 0%), though
             # they were hard to reproduce.  So far it *seems* that in some
@@ -168,45 +184,23 @@ class _CommonAsetekDriver(UsbDriver):
 
         super().disconnect(**kwargs)
 
+    def set_screen(self, channel, mode, value, **kwargs):
+        """Not supported by this device."""
+        raise NotSupportedByDevice()
 
-class Modern690Lc(_CommonAsetekDriver):
-    """Modern fifth generation Asetek 690LC cooler."""
 
-    SUPPORTED_DEVICES = [
-        (0x2433, 0xb200, None, 'Asetek 690LC (assuming EVGA CLC)', {}),
-    ]
+class _ModernBase690Lc(_Base690Lc):
 
-    @classmethod
-    def probe(cls, handle, legacy_690lc=False, **kwargs):
-        if legacy_690lc:
-            return
-        yield from super().probe(handle, **kwargs)
+    def _persist(self):
+        _LOGGER.debug('instructing device to persist current settings')
+        self._begin_transaction()
+        self._write([_CMD_STORE_SETTINGS])
+        self._end_transaction_and_read()
 
-    def downgrade_to_legacy(self):
-        """Take the device handle and return a new Legacy690Lc instance for it.
-
-        This method returns a new instance that takes the device handle from
-        `self`.  Because of this, the caller should immediately discard `self`,
-        as it is no longer valid to call any of its methods or access any of
-        its properties.
-
-        While it is sometimes possible to downgrade a device that has seen
-        modern traffic since it booted, this will generally not work.
-        Additionally, no attempt to disconnect from the device is made while
-        downgrading the instance.
-
-        Thus, callers are strongly advised to only call this function before
-        connecting to the device from this instance and, in fact, before
-        calling any other methods at all on the device, from any instance.
-
-        Finally, this method is not yet considered stable and its signature
-        and/or behavior may change.  Callers should follow the development of
-        liquidctl and the stabilization of this API.
-        """
-        legacy = Legacy690Lc(self.device, self._description)
-        self.device = None
-        self._description = None
-        return legacy
+    def initialize(self, non_volatile=False, **kwargs):
+        super().initialize(**kwargs)
+        if non_volatile:
+            self._persist()
 
     def get_status(self, **kwargs):
         """Get a status report.
@@ -225,9 +219,9 @@ class Modern690Lc(_CommonAsetekDriver):
             ('Firmware version', firmware, '')
         ]
 
-    def set_color(self, channel, mode, colors, time_per_color=1, time_off=None,
-                  alert_threshold=_HIGH_TEMPERATURE, alert_color=[255, 0, 0],
-                  speed=3, **kwargs):
+    def set_color(self, channel, mode, colors, time_per_color=1,
+                  time_off=None, alert_threshold=_HIGH_TEMPERATURE, alert_color=[255, 0, 0],
+                  speed=3, non_volatile=False, **kwargs):
         """Set the color mode for a specific channel."""
         # keyword arguments may have been forwarded from cli args and need parsing
         colors = list(colors)
@@ -261,11 +255,13 @@ class Modern690Lc(_CommonAsetekDriver):
         else:
             raise KeyError(f'unknown lighting mode {mode}')
         self._end_transaction_and_read()
+        if non_volatile:
+            self._persist()
 
-    def set_speed_profile(self, channel, profile, **kwargs):
+    def set_speed_profile(self, channel, profile, non_volatile=False, **kwargs):
         """Set channel to follow a speed duty profile."""
         mtype, dmin, dmax = _VARIABLE_SPEED_CHANNELS[channel]
-        adjusted = self._prepare_profile(profile, dmin, dmax)
+        adjusted = self._prepare_profile(profile, dmin, dmax, _MAX_PROFILE_POINTS)
         for temp, duty in adjusted:
             _LOGGER.info('setting %s PWM point: (%d°C, %d%%), device interpolated',
                          channel, temp, duty)
@@ -273,8 +269,10 @@ class Modern690Lc(_CommonAsetekDriver):
         self._begin_transaction()
         self._write([mtype, 0] + temps + duties)
         self._end_transaction_and_read()
+        if non_volatile:
+            self._persist()
 
-    def set_fixed_speed(self, channel, duty, **kwargs):
+    def set_fixed_speed(self, channel, duty, non_volatile=False, **kwargs):
         """Set channel to a fixed speed duty."""
         if channel == 'fan':
             # While devices seem to recognize a specific channel for fixed fan
@@ -284,7 +282,7 @@ class Modern690Lc(_CommonAsetekDriver):
             # *another* call to initialize(), i.e.  with another
             # configuration command.
             _LOGGER.info('using a flat profile to set %s to a fixed duty', channel)
-            self.set_speed_profile(channel, [(0, duty), (_CRITICAL_TEMPERATURE - 1, duty)])
+            self.set_speed_profile(channel, [(0, duty), (_CRITICAL_TEMPERATURE - 1, duty)], non_volatile, **kwargs)
             return
         mtype, dmin, dmax = _FIXED_SPEED_CHANNELS[channel]
         duty = clamp(duty, dmin, dmax)
@@ -295,20 +293,51 @@ class Modern690Lc(_CommonAsetekDriver):
         self._begin_transaction()
         self._write([mtype, _MIN_PUMP_SPEED_CODE + level])
         self._end_transaction_and_read()
+        if non_volatile:
+            self._persist()
 
+class Modern690Lc(_ModernBase690Lc):
+    """Modern fifth generation Asetek 690LC cooler."""
 
-class Legacy690Lc(_CommonAsetekDriver):
-    """Legacy fifth generation Asetek 690LC cooler."""
-
-    SUPPORTED_DEVICES = [
-        (0x2433, 0xb200, None, 'Asetek 690LC (assuming NZXT Kraken X) (experimental)', {}),
+    _MATCHES = [
+        (0x2433, 0xb200, 'Asetek 690LC (assuming EVGA CLC)', {}),
     ]
 
-    @classmethod
-    def probe(cls, handle, legacy_690lc=False, **kwargs):
-        if not legacy_690lc:
-            return
-        yield from super().probe(handle, **kwargs)
+    def downgrade_to_legacy(self):
+        """Take the device handle and return a new Legacy690Lc instance for it.
+
+        This method returns a new instance that takes the device handle from
+        `self`.  Because of this, the caller should immediately discard `self`,
+        as it is no longer valid to call any of its methods or access any of
+        its properties.
+
+        While it is sometimes possible to downgrade a device that has seen
+        modern traffic since it booted, this will generally not work.
+        Additionally, no attempt to disconnect from the device is made while
+        downgrading the instance.
+
+        Thus, callers are strongly advised to only call this function before
+        connecting to the device from this instance and, in fact, before
+        calling any other methods at all on the device, from any instance.
+
+        Finally, this method is not yet considered stable and its signature
+        and/or behavior may change.  Callers should follow the development of
+        liquidctl and the stabilization of this API.
+        """
+        legacy = Legacy690Lc(self.device, self._description)
+        self.device = None
+        self._description = None
+        return legacy
+
+
+class Legacy690Lc(_Base690Lc):
+    """Legacy fifth generation Asetek 690LC cooler."""
+
+    _MATCHES = [
+        (0x2433, 0xb200, 'Asetek 690LC (assuming NZXT Kraken X)', {}),
+    ]
+
+    _LEGACY_690LC = True
 
     def __init__(self, device, description, **kwargs):
         super().__init__(device, description, **kwargs)
@@ -330,16 +359,17 @@ class Legacy690Lc(_CommonAsetekDriver):
         self._begin_transaction()
         for channel in ['pump', 'fan']:
             mtype, dmin, dmax = _LEGACY_FIXED_SPEED_CHANNELS[channel]
-            duty = clamp(self._data.load_int(f'{channel}_duty', default=dmax), dmin, dmax)
+            duty = clamp(self._data.load(f'{channel}_duty', of_type=int, default=dmax), dmin, dmax)
             _LOGGER.info('setting %s duty to %d%%', channel, duty)
             self._write([mtype, duty])
         return self._end_transaction_and_read()
 
     def initialize(self, **kwargs):
         super().initialize(**kwargs)
-        self._data.store_int('pump_duty', None)
-        self._data.store_int('fan_duty', None)
+        self._data.store('pump_duty', None)
+        self._data.store('fan_duty', None)
         self._set_all_fixed_speeds()
+        _warn_on_unsupported_option(**kwargs)
 
     def get_status(self, **kwargs):
         """Get a status report.
@@ -387,37 +417,33 @@ class Legacy690Lc(_CommonAsetekDriver):
         else:
             raise KeyError(f'unsupported lighting mode {mode}')
         self._end_transaction_and_read()
+        _warn_on_unsupported_option(**kwargs)
 
     def set_fixed_speed(self, channel, duty, **kwargs):
         """Set channel to a fixed speed duty."""
         mtype, dmin, dmax = _LEGACY_FIXED_SPEED_CHANNELS[channel]
         duty = clamp(duty, dmin, dmax)
-        self._data.store_int(f'{channel}_duty', duty)
+        self._data.store(f'{channel}_duty', duty)
         self._set_all_fixed_speeds()
+        _warn_on_unsupported_option(**kwargs)
 
     def set_speed_profile(self, channel, profile, **kwargs):
         """Not supported by this device."""
-        raise NotSupportedByDevice
+        raise NotSupportedByDevice()
 
 
-class Hydro690Lc(Modern690Lc):
+class Hydro690Lc(_ModernBase690Lc):
     """Corsair-branded fifth generation Asetek 690LC cooler."""
 
-    SUPPORTED_DEVICES = [
-        (0x1b1c, 0x0c02, None, 'Corsair Hydro H80i GT (experimental)', {}),
-        (0x1b1c, 0x0c03, None, 'Corsair Hydro H100i GTX (experimental)', {}),
-        (0x1b1c, 0x0c07, None, 'Corsair Hydro H110i GTX (experimental)', {}),
-        (0x1b1c, 0x0c08, None, 'Corsair Hydro H80i v2', {}),
-        (0x1b1c, 0x0c09, None, 'Corsair Hydro H100i v2', {}),
-        (0x1b1c, 0x0c0a, None, 'Corsair Hydro H115i', {}),
+    _MATCHES = [
+        (0x1b1c, 0x0c02, 'Corsair Hydro H80i GT (experimental)', {}),
+        (0x1b1c, 0x0c03, 'Corsair Hydro H100i GTX (experimental)', {}),
+        (0x1b1c, 0x0c07, 'Corsair Hydro H110i GTX (experimental)', {}),
+        (0x1b1c, 0x0c08, 'Corsair Hydro H80i v2', {}),
+        (0x1b1c, 0x0c09, 'Corsair Hydro H100i v2', {}),
+        (0x1b1c, 0x0c0a, 'Corsair Hydro H115i', {}),
     ]
 
-    @classmethod
-    def probe(cls, handle, legacy_690lc=False, **kwargs):
-        # the modern driver overrides probe and rigs it to switch on
-        # --legacy-690lc, so we override it again
-        return super().probe(handle, legacy_690lc=False, **kwargs)
-
     def set_color(self, channel, mode, colors, **kwargs):
         """Set the color mode for a specific channel."""
         if mode == 'rainbow':
@@ -425,6 +451,11 @@ class Hydro690Lc(Modern690Lc):
         super().set_color(channel, mode, colors, **kwargs)
 
 
+def _warn_on_unsupported_option(**kwargs):
+    if kwargs.get('non_volatile'):
+        _LOGGER.warning('device does not support non-volatile settings, skipping')
+
+
 # deprecated aliases
 AsetekDriver = Modern690Lc
 LegacyAsetekDriver = Legacy690Lc
