@@ -31,7 +31,6 @@ class MockCommanderCoreDevice:
 
         self._last_write = bytes()
         self._modes = {}
-
         self._awake = False
 
         self.response_prefix = ()
@@ -52,7 +51,7 @@ class MockCommanderCoreDevice:
         if self._awake:
             if self._last_write[2] == 0x08:  # Get data
                 channel = self._last_write[3]
-                mode = self._modes[channel]
+                mode = self._modes.get(channel)
                 if mode[1] == 0x00:
                     if mode[0] == 0x17:  # Get speeds
                         data.extend([0x06, 0x00])
@@ -111,11 +110,12 @@ class MockCommanderCoreDevice:
         self._last_write = data
         if data[0] != 0x00 or data[1] != 0x08:
             raise ValueError('Start of packets going out should be 00:08')
-
         if data[2] == 0x0d:
             channel = data[3]
-            if self._modes[channel] is None:
+            if  self._modes.get(channel) is None:
                 self._modes[channel] = data[4:6]
+            else:
+                raise ExpectationNotMet('Previous channel was not reset')
         elif data[2] == 0x05 and data[3] == 0x01:
             self._modes[data[4]] = None
         elif data[2] == 0x01 and data[3] == 0x03 and data[4] == 0x00:
@@ -123,7 +123,7 @@ class MockCommanderCoreDevice:
         elif self._awake:
             if data[2] == 0x06:  # Write command
                 channel = data[3]
-                mode = self._modes[channel]
+                mode = self._modes.get(channel)
                 length = u16le_from(data[4:6])
                 data_type = data[8:10]
                 written_data = data[10:8+length]
@@ -143,7 +143,7 @@ class MockCommanderCoreDevice:
 @pytest.fixture
 def commander_core_device():
     device = MockCommanderCoreDevice()
-    core = CommanderCore(device, 'Corsair Commander Core (experimental)')
+    core = CommanderCore(device, 'Corsair Commander Core (experimental)', True)
     core.connect()
     return core
 
@@ -270,16 +270,31 @@ def test_set_fixed_speed_error_commander_core(commander_core_device):
 
 def test_parse_channels_commander_core():
     """This test will go through and thoroughly test CommanderCore._parse_channels so we don't have to in other tests"""
+    core = CommanderCore(MockCommanderCoreDevice(), 'Corsair Commander Core (experimental)', True)
     tests = [
         ('pump', [0]), ('fans', [1, 2, 3, 4, 5, 6]),
         ('fan1', [1]), ('fan2', [2]), ('fan3', [3]), ('fan4', [4]), ('fan5', [5]), ('fan6', [6])
-             ]
+    ]
 
     for (val, answer) in tests:
-        assert list(CommanderCore._parse_channels(val)) == answer
+        assert list(core._parse_channels(val)) == answer
+
+
+
+def test_parse_channels_commander_core_xt():
+    """Test Core XT-specific channel layout"""
+    corext = CommanderCore(MockCommanderCoreDevice(), 'Corsair Commander Core (experimental)', False)
+    tests = [
+        ('fans', [0, 1, 2, 3, 4, 5]),
+        ('fan1', [0]), ('fan2', [1]), ('fan3', [2]), ('fan4', [3]), ('fan5', [4]), ('fan6', [5])
+    ]
+
+    for (val, answer) in tests:
+        assert list(corext._parse_channels(val)) == answer
 
 
 def test_parse_channels_error_commander_core():
     """This tests to make sure we get an error with an invalid channel"""
+    core = CommanderCore(MockCommanderCoreDevice(), 'Corsair Commander Core (experimental)', True)
     with pytest.raises(ValueError):
-        CommanderCore._parse_channels('fan')
+        core._parse_channels('fan')
