@@ -17,7 +17,7 @@ def mock_hwmon(tmp_path):
 
 
 def test_has_module(mock_hwmon):
-    assert mock_hwmon.module == "mock_module"
+    assert mock_hwmon.driver == "mock_module"
 
 
 def test_has_path(mock_hwmon):
@@ -41,4 +41,4 @@ def test_gets_string(mock_hwmon):
 
 
 def test_gets_int(mock_hwmon):
-    assert mock_hwmon.get_int("fan1_input") == 1499
+    assert mock_hwmon.read_int("fan1_input") == 1499
